@@ -241,7 +241,7 @@ class TCPRelayHandler(object):
                         header = b'\x05\x00\x00\x04'
                     else:
                         header = b'\x05\x00\x00\x01'
-                    addr, port = self._local_sock.getsockname()
+                    addr, port = self._local_sock.getsockname()[:2]
                     addr_to_send = socket.inet_pton(self._local_sock.family,
                                                     addr)
                     port_to_send = struct.pack('>H', port)
@@ -407,7 +407,7 @@ class TCPRelayHandler(object):
 
     def _on_local_write(self):
         if self._data_to_write_to_local:
-            data = ''.join(self._data_to_write_to_local)
+            data = b''.join(self._data_to_write_to_local)
             self._data_to_write_to_local = []
             self._write_to_sock(data, self._local_sock)
         else:
